#if !defined( PROCMON_INFO_H )
#if !defined( PROCMON_EXTERN )
/* ---------------------------------------------- */
/* remove typesafe linkage if compiling under c++ */
/* ---------------------------------------------- */
#ifdef __cplusplus
extern "C"
{
#endif
#define PROCMON_EXTERN_ON
#define PROCMON_EXTERN extern
#endif /* !defined( PROCMON_EXTERN ) */
#define PROCMON_INFO_H
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <fcntl.h>
#include <sys/procfs.h>
#include <sys/resource.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <assert.h>
#if defined(_PROCMON_ITIMER)
/* cannot change if using itimer */
#define PROCMON_SIGNAL_NO_ENV
#define PROCMON_SIGNAL_DEFAULT SIGALRM
#else
#define PROCMON_SIGNAL_DEFAULT SIGRTMAX
#endif
#define PROCMON_TAG_DEFAULT "."
#define PROCMON_TAG_LENGTH 32
#define PROCMON_SAMPLE_SAVE 2
#define PROCMON_MAX_LEN 1024
/* procmon_info_get return: likely never fill with data (eg process gone) */
#define PROCMON_ERROR 1
/* procmon_info_get return: current sampling suspect and pinfo not filled */
#define PROCMON_WARNING 2
/* ==========================================================================
 * Copy:
 *    PROCMON_info_copy( PROCMON_INFO_struct *pinfo_orig,
 *                       PROCMON_INFO_struct *pinfo_copy )
 * Fill:
 *    PROCMON_info_get( long long pid,
 *                      PROCMON_INFO_struct *pinfo )
 * Init:
 *    PROCMON_info_init( PROCMON_INFO_struct *pinfo )
 *
 * Print:
 *    PROCMON_info_print( PROCMON_INFO_struct *pinfo,
 *                        PROCMON_INFO_SETTINGS_struct *psettings,
 *                        FILE *outfile )
 *
 * Usage:
 * ------
 * Link in libprocmon_info.a
 * C code sample:
 *    #include "procmon_info.h"
 *    ...<in some routine>...
 *    PROCMON_INFO_struct pinfo
 *    PROCMON_info_init( &pinfo ); // initialize
 *    PROCMON_info_get( 0, &pinfo ); // get info about this process
 *    PROCMON_info_print( &pinfo, NULL, NULL ); // print pinfo struct to stdout
 *    ...<some more code>...
 *    PROCMON_info_get( 0, &pinfo ); // get another sampling (use same pinfo)
 *    PROCMON_info_print( &pinfo, NULL, NULL ); // print pinfo
 *
 * Notes
 * -----
 * o Units and Datatypes
 *   Times are in seconds.  Sizes are in bytes.
 *   Use simple data types (double, long long)
 *    - Simpler printing of values as doubles.
 *    - Easier future(?) translation to Fortran.
 *     (no unsigned long long or variable sized long)
 *    - Struct only contains 8-byte members to avoid padding issues when
 *      mixing compilers.
 *    - EpochTime will still have about 6 digits of seconds precision
 *      (usecs) - which is relly the same precision as the clock I am using
 *      anyway.
 * o Specialized Use
 *   Users might wish to call PROCMON_info_get() at certain points in
 *   their code to get information at interesting times or to gracefully
 *   terminate if, for example, the amount of free machine memory drops
 *   below 10%.
 * ======================================================================== */
typedef struct PROCMON_INFO_struct
{
  long long num_samples; /* number of samplings pinfo has successfully done */
  long long num_samples_printed; /* number of times this pinfo was printed */
  long long printed_header; /* if already printed the header
                             * Set to !0 if never want to print header */
  long long chars_printed; /* the number of characters printed */
  double wall;        /* secs EpochTime - for getting common relative time */
  /* machine (m) info */
  double m_size;      /* total memory in bytes */
  double m_free;      /* free memory in bytes */
  /* static process (p) info */
  long long p_pid;    /* process id */
  long long p_ppid;   /* parent process id */
  long long p_uid;    /* user ID of process */
  char p_command[PROCMON_MAX_LEN]; /* process command */
  long long thread;         /* if this is a thread or not (not implemented) */
  double p_size_lim;  /* byte size limit (<=0 if unknown) */
  double p_rss_lim;   /* byte rss size limit (<=0 if unknown) */
  double p_data_lim;  /* byte data size limit (<=0 if unknown) */
  double p_stack_lim; /* byte stack size limit (<=0 if unknown) */
  /* dynamic process (p) info */
  double p_size[PROCMON_SAMPLE_SAVE]; /* byte process size [cur, max] */
  double p_rss;       /* byte process resident set size */
  double p_data;      /* byte process data size */
  double p_stack;     /* byte process stack size */
  double p_time[PROCMON_SAMPLE_SAVE]; /* sec proc time [current, previous] */
  double p_wall[PROCMON_SAMPLE_SAVE]; /* sec proc wall clock time since this
                                         process started: [cur, previous] */
  double p_majflt;    /* number of major page faults - disk reads */
  double p_minflt;    /* number of minor page faults - no disk reads */
}
PROCMON_INFO_struct;
/* ==========================================================================
 * Fill:
 *    PROCMON_info_settings_get( int argc,
 *                               const char **argv,
 *                               PROCMON_INFO_SETTINGS_struct *psettings )
 *
 * Print:
 *    PROCMON_info_settings_print( int argc,
 *                                 const char **argv,
 *                                 PROCMON_INFO_SETTINGS_struct *psettings )
 *
 * Usage:
 * ------
 * Link in libprocmon_info.a
 * C code sample:
 *    #include "procmon_info.h"
 *    ...<in some routine>...
 *    PROCMON_INFO_SETTINGS_struct psettings;
 *    PROCMON_info_settings_get( &psettings ); // fill settings struct
 *    PROCMON_info_settings_print( NULL, &psettings ); // print struct - stdout
 *
 * Notes
 * -----
 * This routine is used by the procmon internals to get environment
 * variable settings (and print them in the header of the output).  It
 * is also used by the standalone executable procmon to parse the command
 * line options.  It might be useful to other users as well (eg for getting
 * env var settings and creating outfile_name).
 * ======================================================================== */
typedef struct PROCMON_INFO_SETTINGS_struct
{
  /* ---------------------------------- */
  /* env vars:          PROCMON_<NAME>  */
  /* command line args: -<name> <value> */
  /* ---------------------------------- */
  long long
    time,                 /* time (milliseconds) between samplings */
    interval,             /* print every n-th sample  */
    min;                  /* minimum number of samples to keep file */
  char
    dir[PROCMON_MAX_LEN]; /* directory setting */
  /* ------------ */
  /* other things */
  /* ------------ */
  long long
    all,                  /* if getting info about all processes */
    pid,                  /* process id to get info about (0) */
    pretty,               /* whether or not to try and do pretty output (0) */
    prettyw,              /* screen width (used when cannot get width) */
    signal,               /* signal to use for sampling trigger */
    uid;                  /* process uid to get info about (-1) */
  char
    tag[PROCMON_TAG_LENGTH]; /* tag name used in printing */
  char
    command[PROCMON_MAX_LEN], /* command to get info about
                               * strlen will be 0 if not set */
    outfile_name[PROCMON_MAX_LEN], /* outfile_prefix + <pid>.txt
                                    * The pid part of the name is assumed to
                                    * be the pid of the process (or the pid
                                    * specified on the command line).  If
                                    * you are using command line arguments
                                    * that specify UID/command, this name will
                                    * be incorrect and you should make your
                                    * own name.
                                    * strlen will be 0 for stdout. */
    outfile_prefix[PROCMON_MAX_LEN]; /* output file name prefix:
                                      *   <dir>/procmon_<hostname>_
                                      * Full file name will have suffix:
                                      *   <pid>.txt
                                      * strlen will be 0 for stdout */
}
PROCMON_INFO_SETTINGS_struct;
/* ------------------------------------------------------------------ */
/* structure holding generic data and sample data used by procmon_rld */
/* ------------------------------------------------------------------ */
typedef struct
{
  FILE
    *filename_fp; /* file handle to output file */
  int
    chain_signal, /* whether or not to chain to other signal handlers */
    filename_fd,  /* file descriptor gotten from mkstemp command */
    in_handler,   /* if in sig handler */
    initialized;  /* if this has been initialized or not */
  PROCMON_INFO_SETTINGS_struct
    psettings;    /* command line, env var, ... */
  PROCMON_INFO_struct
    pinfo;        /* collection of the sample data */
  /* ------------ */
  /* signal stuff */
  /* ------------ */
  struct sigaction
    action,         /* new action */
    action_ignore,  /* when setting to ignore the signal */
    action_old;     /* old action */
  int
    which_sigaction; /* sent to sigaction */
#if defined(_PROCMON_ITIMER)
  /* ------------ */
  /* itimer stuff */
  /* ------------ */
  int
    which_itimer; /* sent to setitimer */
  struct itimerval
    value,        /* new value */
    value_old;    /* old value */
#else
  /* ------------------ */
  /* POSIX4 timer stuff */
  /* ------------------ */
  timer_t
    psxtimer;    /* POSIX4 timer */
  struct sigevent
    psxsigevent; /* POSIX4 signal event */
  struct itimerspec
    psxspec,     /* new value */
    psxspec_old; /* old value */
#endif
}
PROCMON_struct;
PROCMON_EXTERN PROCMON_struct procmon_s;
/* ======================================================================= */
/* ============================== PROTOTYPES ============================= */
/* ======================================================================= */
/*
 * PROCMON_info_get_print - simplified wrap interface to get an info struct
 *                          and then print it.
 *                            PROCMON_info_get_print( "foo", 0 );
 *                            foo();
 *                            PROCMON_info_get_print( "foo", 1 );
 *   tag:            Name of the sampling
 *   state:          0 = start, 1 = stop
 */
void PROCMON_info_get_print(
                            const char *tag,
                            int state
                            );
void procmon_info_get_print(
                            const char *tag,
                            int *state
                            );
void procmon_info_get_print_(
                             const char *tag,
                             int *state
                             );
void procmon_info_get_print__(
                              const char *tag,
                              int *state
                              );
void PROCMON_INFO_GET_PRINT(
                            const char *tag,
                            int *state
                            );
/*
 * PROCMON_info_copy - create copy of pinfo struct
 *   pinfo:  original
 *   pinfoc: copy
 */
int PROCMON_info_copy(
                      const PROCMON_INFO_struct *pinfo,
                      PROCMON_INFO_struct *pinfoc
                      );
/*
 * PROCMON_info_get - fills pinfo struct (call PROCMON_info_init first)
 *   pid:   process ID (use 0 for current process)
 *   pinfo: process info struct to fill
 */
int PROCMON_info_get(
                      long long pid,
                      PROCMON_INFO_struct *pinfo
                      );
/*
 * PROCMON_info_init - initializes pinfo struct
 *   pinfo: process info struct
 */
int PROCMON_info_init(
                      PROCMON_INFO_struct *pinfo
                      );
/*
 * PROCMON_info_print - print a pinfo struct
 *   pinfo:   process info struct
 *   outfile: file pointer (if NULL, stdout will be used)
 */
int PROCMON_info_print(
                       PROCMON_INFO_struct *pinfo,
                       PROCMON_INFO_SETTINGS_struct *psettings,
                       FILE *outfile
                       );
/*
 * PROCMON_info_settings_get - get env and command line settings
 *   argc:           from main( argc, argv ) - if 0,    only uses env vars
 *   argv:           from main( argc, argv ) - if NULL, only uses env vars
 *   psettings: struct to fill
 */
int PROCMON_info_settings_get(
                              int argc,
                              const char **argv,
                              PROCMON_INFO_SETTINGS_struct *psettings
                              );
/*
 * PROCMON_info_settings_print - print settings struct
 *   argc:           from main( argc, argv ) - if 0,    only uses env vars
 *   argv:           from main( argc, argv ) - if NULL, only uses env vars
 *   psettings: struct to fill
 */
int PROCMON_info_settings_print(
                                const PROCMON_INFO_SETTINGS_struct *psettings,
                                FILE *outfile
                                );
/*
 * PROCMON_rld_handler - signal handler of PROCMON_rld.
 *  Can be called manually to print out data - should probably use
 *  PROCMON_info_get_print() instead as this is a lower level routine.
 *   sig:            Signal (set to 0 if calling manually)
 *   info:           Required field (set to NULL if calling manually)
 *   context:        Required field (set to NULL if calling manually)
 */
void PROCMON_rld_handler(
                         int sig,
                         siginfo_t *info,
                         void *context
                         );
/*
 * PROCMON_rld_reset - This resets the automatic sampling environment
 *                     after a fork() is called.  Some MPI implementations
 *                     use fork() to spawn processes.  Various things
 *                     need to be reset (eg a new output file and a
 *                     new signal timer).  If you do not call this
 *                     after the fork() (eg after the MPI_Init() call) when
 *                     doing automatic sampling, the child process(es) will
 *                     not do any automatic sampling until the last
 *                     sampling at program termination (note, any manual
 *                     sampling will still be done).
 *   no arguments
 */
void PROCMON_rld_reset(
                       void
                       );
void procmon_rld_reset(
                       void
                       );
void procmon_rld_reset_(
                        void
                        );
void procmon_rld_reset__(
                         void
                         );
void PROCMON_RLD_RESET(
                       void
                       );
/* ----------------- */
/* internal routines */
/* ----------------- */
int PROCMON_init_file(
                      PROCMON_INFO_struct *pinfo,
                      PROCMON_INFO_SETTINGS_struct *psettings,
                      FILE **fp
                      );
void PROCMON_rld_init_timer(
                            void
                            );
void PROCMON_rld_handler_noop(
                              int sig,
                              siginfo_t *info,
                              void *context
                              );
/* ---------------------------------------------- */
/* remove typesafe linkage if compiling under c++ */
/* ---------------------------------------------- */
#ifdef __cplusplus
}
#endif
#endif /* PROCMON_INFO_H */
